@@ -18,10 +18,12 @@ import (
 	"strings"
 	"syscall"
 
+	"github.com/google/go-containerregistry/pkg/authn"
 	"github.com/spf13/cobra"
 	keyclient "github.com/sylabs/scs-key-client/client"
 	"github.com/sylabs/singularity/v4/internal/pkg/build"
 	"github.com/sylabs/singularity/v4/internal/pkg/build/args"
+	bkclient "github.com/sylabs/singularity/v4/internal/pkg/build/buildkit/client"
 	"github.com/sylabs/singularity/v4/internal/pkg/build/remotebuilder"
 	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
 	"github.com/sylabs/singularity/v4/internal/pkg/cache"
@@ -111,30 +113,45 @@ func runBuild(cmd *cobra.Command, args []string) {
 		if buildArgs.remote {
 			sylog.Fatalf("--nv option is not supported for remote build")
 		}
+		if isOCI {
+			sylog.Fatalf("--nv option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_NV", "1")
 	}
 	if buildArgs.nvccli {
 		if buildArgs.remote {
 			sylog.Fatalf("--nvccli option is not supported for remote build")
 		}
+		if isOCI {
+			sylog.Fatalf("--nvccli option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_NVCCLI", "1")
 	}
 	if buildArgs.rocm {
 		if buildArgs.remote {
 			sylog.Fatalf("--rocm option is not supported for remote build")
 		}
+		if isOCI {
+			sylog.Fatalf("--rocm option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_ROCM", "1")
 	}
 	if len(buildArgs.bindPaths) > 0 {
 		if buildArgs.remote {
 			sylog.Fatalf("-B/--bind option is not supported for remote build")
 		}
+		if isOCI {
+			sylog.Fatalf("-B/--bind option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_BINDPATH", strings.Join(buildArgs.bindPaths, ","))
 	}
 	if len(buildArgs.mounts) > 0 {
 		if buildArgs.remote {
 			sylog.Fatalf("--mount option is not supported for remote build")
 		}
+		if isOCI {
+			sylog.Fatalf("--mount option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_MOUNT", strings.Join(buildArgs.mounts, "\n"))
 	}
 	if buildArgs.writableTmpfs {
@@ -144,10 +161,17 @@ func runBuild(cmd *cobra.Command, args []string) {
 		if buildArgs.fakeroot {
 			sylog.Fatalf("--writable-tmpfs option is not supported for fakeroot build")
 		}
+		if isOCI {
+			sylog.Fatalf("--writable-tmpfs option is not supported for OCI builds from Dockerfiles")
+		}
 		os.Setenv("SINGULARITY_WRITABLE_TMPFS", "1")
 	}
 
-	if buildArgs.arch != runtime.GOARCH && !buildArgs.remote {
+	if cmd.Flags().Lookup("authfile").Changed && buildArgs.remote {
+		sylog.Fatalf("Custom authfile is not supported for remote build")
+	}
+
+	if buildArgs.arch != runtime.GOARCH && !(buildArgs.remote || isOCI) {
 		sylog.Fatalf("Requested architecture (%s) does not match host (%s). Cannot build locally.", buildArgs.arch, runtime.GOARCH)
 	}
 
@@ -155,7 +179,7 @@ func runBuild(cmd *cobra.Command, args []string) {
 	spec := args[1]
 
 	// Non-remote build with def file as source
-	rootNeeded := !buildArgs.remote && fs.IsFile(spec) && !isImage(spec)
+	rootNeeded := !buildArgs.remote && fs.IsFile(spec) && !isImage(spec) && !isOCI
 
 	if rootNeeded && syscall.Getuid() != 0 && !buildArgs.fakeroot {
 		prootPath, err := bin.FindBin("proot")
@@ -173,9 +197,41 @@ func runBuild(cmd *cobra.Command, args []string) {
 
 	if buildArgs.remote {
 		runBuildRemote(cmd.Context(), cmd, dest, spec)
-	} else {
-		runBuildLocal(cmd.Context(), cmd, dest, spec)
+		return
 	}
+
+	authConf, err := makeOCICredentials(cmd)
+	if err != nil {
+		sylog.Fatalf("While creating Docker credentials: %v", err)
+	}
+
+	if isOCI {
+		reqArch := ""
+		if cmd.Flags().Lookup("arch").Changed {
+			reqArch = buildArgs.arch
+		}
+		wd, err := os.Getwd()
+		if err != nil {
+			sylog.Fatalf("While trying to determine current dir: %v", err)
+		}
+
+		bkOpts := &bkclient.Opts{
+			AuthConf:        authConf,
+			ReqAuthFile:     reqAuthFile,
+			BuildVarArgs:    buildArgs.buildVarArgs,
+			BuildVarArgFile: buildArgs.buildVarArgFile,
+			ReqArch:         reqArch,
+			KeepLayers:      keepLayers,
+			ContextDir:      wd,
+			DisableCache:    disableCache,
+		}
+		if err := bkclient.Run(cmd.Context(), bkOpts, dest, spec); err != nil {
+			sylog.Fatalf("%v", err)
+		}
+	} else {
+		runBuildLocal(cmd.Context(), authConf, cmd, dest, spec)
+	}
+
 	sylog.Infof("Build complete: %s", dest)
 }
 
@@ -303,7 +359,7 @@ func runBuildRemote(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 	}
 }
 
-func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
+func runBuildLocal(ctx context.Context, authConf *authn.AuthConfig, cmd *cobra.Command, dst, spec string) {
 	var keyInfo *cryptkey.KeyInfo
 	if buildArgs.encrypt || promptForPassphrase || cmd.Flags().Lookup("pem-path").Changed {
 		if os.Getuid() != 0 {
@@ -333,11 +389,6 @@ func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 		sylog.Fatalf("Could not check build sections: %v", err)
 	}
 
-	authConf, err := makeDockerCredentials(cmd)
-	if err != nil {
-		sylog.Fatalf("While creating Docker credentials: %v", err)
-	}
-
 	// parse definition to determine build source
 	buildArgsMap, err := args.ReadBuildArgs(buildArgs.buildVarArgs, buildArgs.buildVarArgFile)
 	if err != nil {
@@ -418,8 +469,9 @@ func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 				LibraryURL:        buildArgs.libraryURL,
 				LibraryAuthToken:  authToken,
 				KeyServerOpts:     ko,
-				DockerAuthConfig:  authConf,
+				OCIAuthConfig:     authConf,
 				DockerDaemonHost:  dockerHost,
+				DockerAuthFile:    reqAuthFile,
 				EncryptionKeyInfo: keyInfo,
 				FixPerms:          buildArgs.fixPerms,
 				SandboxTarget:     sandboxTarget,
