@@ -774,7 +774,11 @@ func (c actionTests) PersistentOverlay(t *testing.T) {
 	}
 
 	// create the squashfs overlay image
-	cmd := exec.Command("mksquashfs", squashDir, squashfsImage, "-noappend", "-all-root")
+	mksquashfsCmd, err := bin.FindBin("mksquashfs")
+	if err != nil {
+		t.Fatalf("Unable to find 'mksquashfs' binary even though require.Command() was called: %v", err)
+	}
+	cmd := exec.Command(mksquashfsCmd, squashDir, squashfsImage, "-noappend", "-all-root")
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
@@ -1762,10 +1766,11 @@ func (c actionTests) fuseMount(t *testing.T) {
 	}
 
 	basicTests := []struct {
-		name    string
-		spec    string
-		key     string
-		profile e2e.Profile
+		name         string
+		spec         string
+		key          string
+		profile      e2e.Profile
+		requirements func(t *testing.T)
 	}{
 		{
 			name:    "HostDaemonAsRoot",
@@ -1820,48 +1825,72 @@ func (c actionTests) fuseMount(t *testing.T) {
 			spec:    "host-daemon",
 			key:     userPrivKey,
 			profile: e2e.UserNamespaceProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "HostAsUserNamespace",
 			spec:    "host",
 			key:     userPrivKey,
 			profile: e2e.UserNamespaceProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "ContainerDaemonAsUserNamespace",
 			spec:    "container-daemon",
 			key:     userPrivKey,
 			profile: e2e.UserNamespaceProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "ContainerAsUserNamespace",
 			spec:    "container",
 			key:     userPrivKey,
 			profile: e2e.UserNamespaceProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "HostDaemonAsFakeroot",
 			spec:    "host-daemon",
 			key:     userPrivKey,
 			profile: e2e.FakerootProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "HostAsFakeroot",
 			spec:    "host",
 			key:     userPrivKey,
 			profile: e2e.FakerootProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "ContainerDaemonAsFakeroot",
 			spec:    "container-daemon",
 			key:     userPrivKey,
 			profile: e2e.FakerootProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 		{
 			name:    "ContainerAsFakeroot",
 			spec:    "container",
 			key:     userPrivKey,
 			profile: e2e.FakerootProfile,
+			requirements: func(t *testing.T) {
+				require.Kernel(t, 4, 18)
+			},
 		},
 	}
 
@@ -1871,6 +1900,7 @@ func (c actionTests) fuseMount(t *testing.T) {
 	for _, tt := range basicTests {
 		c.env.RunSingularity(
 			t,
+			e2e.PreRun(tt.requirements),
 			e2e.AsSubtest(tt.name),
 			e2e.WithProfile(tt.profile),
 			e2e.WithCommand("exec"),
@@ -1934,7 +1964,11 @@ func (c actionTests) bindImage(t *testing.T) {
 	}
 
 	// create the squashfs overlay image
-	cmd := exec.Command("mksquashfs", squashDir, squashfsImage, "-noappend", "-all-root")
+	mksquashfsCmd, err := bin.FindBin("mksquashfs")
+	if err != nil {
+		t.Fatalf("Unable to find 'mksquashfs' binary even though require.Command() was called: %v", err)
+	}
+	cmd := exec.Command(mksquashfsCmd, squashDir, squashfsImage, "-noappend", "-all-root")
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
@@ -2507,73 +2541,93 @@ func (c actionTests) actionCompat(t *testing.T) {
 	}
 }
 
-// actionSquashfuse tests that experimental unpriv squashfuse SIF mount works.
-func (c actionTests) actionSIFFUSE(t *testing.T) {
+// actionFUSEImage tests that squashfuse SIF mount works. Currently forced here
+// via deprecated `--sif-fuse` flag as this is convenient to include non-userns
+// profiles without changing global config.
+func (c actionTests) actionFUSEImage(t *testing.T) {
 	require.Command(t, "squashfuse")
 	require.Command(t, "fusermount")
 	e2e.EnsureImage(t, c.env)
 
 	beforeCount := countSquashfuseMounts(t)
 
-	c.env.RunSingularity(
-		t,
-		e2e.WithProfile(e2e.UserNamespaceProfile),
-		e2e.WithCommand("exec"),
-		e2e.WithArgs("--sif-fuse", c.env.ImagePath, "/bin/true"),
-		e2e.ExpectExit(
-			0,
-			e2e.ExpectError(e2e.ContainMatch, "Mounting SIF with FUSE"),
-			e2e.ExpectError(e2e.ContainMatch, "Unmounting SIF with FUSE"),
-		),
-	)
+	for _, p := range e2e.NativeProfiles {
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(p.String()),
+			e2e.WithProfile(e2e.UserNamespaceProfile),
+			e2e.WithCommand("exec"),
+			e2e.WithGlobalOptions("-d"),
+			e2e.WithArgs("--sif-fuse", c.env.ImagePath, "ps"),
+			e2e.ExpectExit(
+				0,
+				e2e.ExpectOutput(e2e.ContainMatch, "squashfuse"),
+				e2e.ExpectError(e2e.ContainMatch, "Mounting image with FUSE"),
+				e2e.ExpectError(e2e.ContainMatch, "PostStartHost()"),
+				e2e.ExpectError(e2e.ContainMatch, "CleanupHost()"),
+			),
+		)
 
-	afterCount := countSquashfuseMounts(t)
-	if afterCount != beforeCount {
-		t.Errorf("found %d squashfuse mounts before execution, and %d remaining after", beforeCount, afterCount)
+		afterCount := countSquashfuseMounts(t)
+		if afterCount != beforeCount {
+			t.Errorf("found %d squashfuse mounts before execution, and %d remaining after", beforeCount, afterCount)
+		}
 	}
 }
 
-// Verify that the FUSE mounts, and the CleanupHost() process are not seen when
-// --sif-fuse should not be in effect.
-func (c actionTests) actionNoSIFFUSE(t *testing.T) {
+// Verify that the FUSE mounts, and the PostStartHost/CleanupHost() processes are not seen when
+// FUSE mounts of a SIF image should not be in effect.
+func (c actionTests) actionNoFUSEImage(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
-	profiles := []e2e.Profile{e2e.UserProfile, e2e.RootProfile, e2e.FakerootProfile, e2e.UserNamespaceProfile}
-
-	for _, p := range profiles {
+	for _, p := range []e2e.Profile{e2e.RootProfile, e2e.UserProfile} {
 		c.env.RunSingularity(
 			t,
 			e2e.AsSubtest(p.String()),
 			e2e.WithProfile(p),
 			e2e.WithCommand("exec"),
 			e2e.WithGlobalOptions("-d"),
-			e2e.WithArgs(c.env.ImagePath, "mount"),
+			e2e.WithArgs(c.env.ImagePath, "ps"),
 			e2e.ExpectExit(
 				0,
 				e2e.ExpectError(e2e.UnwantedContainMatch, "squashfuse"),
+				e2e.ExpectError(e2e.UnwantedContainMatch, "PostStartHost()"),
 				e2e.ExpectError(e2e.UnwantedContainMatch, "CleanupHost()"),
 			),
 		)
 	}
 }
 
-// actionTmpSandboxFlag tests the command-line option prohibiting unpacking of image
+// actionTmpSandboxFlag tests the command-line options forcing / prohibiting unpacking of image
 // files into temporary sandbox dirs.
 func (c actionTests) actionTmpSandboxFlag(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
-	profiles := []e2e.Profile{e2e.UserProfile, e2e.RootProfile, e2e.FakerootProfile, e2e.UserNamespaceProfile}
-
-	for _, p := range profiles {
-		c.env.RunSingularity(
-			t,
-			e2e.AsSubtest(p.String()),
-			e2e.WithProfile(p),
-			e2e.WithCommand("exec"),
-			e2e.WithArgs("--sif-fuse=false", "--no-tmp-sandbox", "-u", c.env.ImagePath, "/bin/true"),
-			e2e.ExpectExit(255),
-		)
+	// --tmp-sandbox should override kernel mount (setuid profiles) and squashfuse mount (userns profiles).
+	for _, p := range e2e.NativeProfiles {
+		t.Run(p.String(), func(t *testing.T) {
+			c.env.RunSingularity(
+				t,
+				e2e.AsSubtest("tmp-sandbox"),
+				e2e.WithProfile(p),
+				e2e.WithCommand("exec"),
+				e2e.WithArgs("--tmp-sandbox", c.env.ImagePath, "/bin/sh", "-c", "echo $SINGULARITY_CONTAINER"),
+				e2e.ExpectExit(0,
+					e2e.ExpectOutput(e2e.RegexMatch, `/rootfs-(\d+)/root`), // <tmpdir>/rootfs-xxxxxxxxx/root
+					e2e.ExpectError(e2e.ContainMatch, "Converting SIF file to temporary sandbox"),
+				),
+			)
+		})
 	}
+
+	c.env.RunSingularity(
+		t,
+		e2e.AsSubtest("no-tmp-sandbox"),
+		e2e.WithProfile(e2e.UserProfile),
+		e2e.WithCommand("exec"),
+		e2e.WithArgs("--tmp-sandbox", "--no-tmp-sandbox", c.env.ImagePath, "/bin/sh", "-c", "echo $SINGULARITY_CONTAINER"),
+		e2e.ExpectExit(255),
+	)
 }
 
 // Make sure --workdir and --scratch work together nicely even when workdir is a
@@ -2656,6 +2710,134 @@ func (c actionTests) actionNoSetgroups(t *testing.T) {
 	)
 }
 
+// actionAuth tests run/exec/shell flows that involve authenticated pulls from
+// OCI registries.
+func (c actionTests) actionAuth(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	profiles := []e2e.Profile{
+		e2e.UserProfile,
+		e2e.RootProfile,
+		e2e.OCIUserProfile,
+		e2e.OCIRootProfile,
+	}
+
+	for _, p := range profiles {
+		t.Run(p.String(), func(t *testing.T) {
+			t.Run("default", func(t *testing.T) {
+				c.actionAuthTester(t, false, p)
+			})
+			t.Run("custom", func(t *testing.T) {
+				c.actionAuthTester(t, true, p)
+			})
+		})
+	}
+}
+
+func (c actionTests) actionAuthTester(t *testing.T, withCustomAuthFile bool, profile e2e.Profile) {
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, c.env.TestDir, "action-auth", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	prevCwd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("could not get current working directory: %s", err)
+	}
+	defer os.Chdir(prevCwd)
+	if err = os.Chdir(tmpdir); err != nil {
+		t.Fatalf("could not change cwd to %q: %s", tmpdir, err)
+	}
+
+	localAuthFileName := ""
+	if withCustomAuthFile {
+		localAuthFileName = "./my_local_authfile"
+	}
+
+	authFileArgs := []string{}
+	if withCustomAuthFile {
+		authFileArgs = []string{"--authfile", localAuthFileName}
+	}
+
+	t.Cleanup(func() {
+		e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+	})
+
+	orasCustomPushTarget := fmt.Sprintf(
+		"oras://%s/authfile-%s-oras-alpine:latest",
+		c.env.TestRegistryPrivPath, strings.ToLower(profile.String()),
+	)
+
+	tests := []struct {
+		name          string
+		cmd           string
+		args          []string
+		whileLoggedIn bool
+		expectExit    int
+	}{
+		{
+			name:          "docker before auth",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "docker",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+		{
+			name:          "noauth docker",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "oras push",
+			cmd:           "push",
+			args:          []string{c.env.ImagePath, orasCustomPushTarget},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+		{
+			name:          "noauth oras",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", orasCustomPushTarget, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "oras",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", orasCustomPushTarget, "true"},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+	}
+
+	for _, tt := range tests {
+		if tt.whileLoggedIn {
+			e2e.PrivateRepoLogin(t, c.env, profile, localAuthFileName)
+		} else {
+			e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+		}
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(profile),
+			e2e.WithCommand(tt.cmd),
+			e2e.WithArgs(append(authFileArgs, tt.args...)...),
+			e2e.ExpectExit(tt.expectExit),
+		)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := actionTests{
@@ -2692,6 +2874,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"issue 5631":                   c.issue5631,                      // https://github.com/sylabs/singularity/issues/5631
 		"issue 5690":                   c.issue5690,                      // https://github.com/sylabs/singularity/issues/5690
 		"issue 1950":                   c.issue1950,                      // https://github.com/sylabs/singularity/issues/1950
+		"issue 2690":                   c.issue2690,                      // https://github.com/sylabs/singularity/issues/2690
 		"network":                      c.actionNetwork,                  // test basic networking
 		"binds":                        c.actionBinds,                    // test various binds with --bind and --mount
 		"exit and signals":             c.exitSignals,                    // test exit and signals propagation
@@ -2702,11 +2885,12 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"compat":                       np(c.actionCompat),               // test --compat
 		"umask":                        np(c.actionUmask),                // test umask propagation
 		"invalidRemote":                np(c.invalidRemote),              // GHSA-5mv9-q7fq-9394
-		"SIFFUSE":                      np(c.actionSIFFUSE),              // test --sif-fuse
-		"NoSIFFUSE":                    np(c.actionNoSIFFUSE),            // test absence of squashfs and CleanupHost()
-		"TmpSandboxFlag":               c.actionTmpSandboxFlag,           // test --no-tmp-sandbox flag
+		"FUSEImage":                    np(c.actionFUSEImage),            // test explicit FUSE image mount
+		"NoFUSEImage":                  np(c.actionNoFUSEImage),          // test absence of squashfuse and CleanupHost()
+		"TmpSandboxFlag":               c.actionTmpSandboxFlag,           // test --tmp-sandbox / --no-tmp-sandbox flag
 		"relWorkdirScratch":            np(c.relWorkdirScratch),          // test relative --workdir with --scratch
 		"ociRelWorkdirScratch":         np(c.actionOciRelWorkdirScratch), // test relative --workdir with --scratch in OCI mode
+		"auth":                         np(c.actionAuth),                 // tests action cmds w/authenticated pulls from OCI registries
 		//
 		// OCI Runtime Mode
 		//
