@@ -17,6 +17,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 	"strings"
 	"testing"
 
@@ -28,8 +29,10 @@ import (
 	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
 	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
 	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/tmpl"
 	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
 	"golang.org/x/sys/unix"
+	"gotest.tools/assert"
 )
 
 type ctx struct {
@@ -237,21 +240,21 @@ func (c ctx) testDockerHost(t *testing.T) {
 					c.env.RunSingularity(t, cmdOps...)
 				}
 			})
+		})
 
-			t.Run("build", func(t *testing.T) {
-				for _, tt := range tests {
-					cmdOps := []e2e.SingularityCmdOp{
-						e2e.WithProfile(profile),
-						e2e.AsSubtest(tt.name),
-						e2e.WithCommand("build"),
-						e2e.WithArgs("--force", "--disable-cache", "test.sif", dockerURI),
-						e2e.WithEnv(append(os.Environ(), tt.envarName+"="+tt.envarValue)),
-						e2e.WithDir(tmpPath),
-						e2e.ExpectExit(tt.exit),
-					}
-					c.env.RunSingularity(t, cmdOps...)
+		t.Run("build", func(t *testing.T) {
+			for _, tt := range tests {
+				cmdOps := []e2e.SingularityCmdOp{
+					e2e.WithProfile(e2e.RootProfile),
+					e2e.AsSubtest(tt.name),
+					e2e.WithCommand("build"),
+					e2e.WithArgs("--force", "--disable-cache", "test.sif", dockerURI),
+					e2e.WithEnv(append(os.Environ(), tt.envarName+"="+tt.envarValue)),
+					e2e.WithDir(tmpPath),
+					e2e.ExpectExit(tt.exit),
 				}
-			})
+				c.env.RunSingularity(t, cmdOps...)
+			}
 		})
 	}
 
@@ -266,8 +269,311 @@ func (c ctx) testDockerHost(t *testing.T) {
 	})(t)
 }
 
-// AUFS sanity tests
+// Test that DOCKER_xyz env vars take priority over other means of
+// authenticating with Docker - in particular, the --authfile flag.
+func (c ctx) testDockerCredsPriority(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	privImgNoPrefix := strings.TrimPrefix(c.env.TestRegistryPrivImage, "docker://")
+	simpleDef := e2e.PrepareDefFile(e2e.DefFileDetails{
+		Bootstrap: "docker",
+		From:      privImgNoPrefix,
+	})
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(simpleDef)
+		}
+	})
+
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, c.env.TestDir, "build-auth", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	dockerfileContent := fmt.Sprintf(
+		`
+FROM %s
+CMD /bin/true
+`,
+		privImgNoPrefix,
+	)
+	dockerfile, err := e2e.WriteTempFile(tmpdir, "Dockerfile", dockerfileContent)
+	if err != nil {
+		t.Fatalf("while trying to generate test dockerfile: %v", err)
+	}
+	ocisifPath := dockerfile + ".oci.sif"
+
+	profiles := []e2e.Profile{
+		e2e.UserProfile,
+		e2e.RootProfile,
+	}
+
+	for _, p := range profiles {
+		t.Run(p.String(), func(t *testing.T) {
+			t.Run("def pull", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, false, p, "pull", "--disable-cache", "--no-https", "-F", c.env.TestRegistryPrivImage)
+			})
+			t.Run("def exec", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, false, p, "exec", "--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true")
+			})
+			t.Run("cstm pull", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, true, p, "pull", "--disable-cache", "--no-https", "-F", c.env.TestRegistryPrivImage)
+			})
+			t.Run("cstm exec", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, true, p, "exec", "--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true")
+			})
+		})
+	}
+	profiles = []e2e.Profile{
+		e2e.OCIUserProfile,
+		e2e.OCIRootProfile,
+	}
+
+	for _, p := range profiles {
+		t.Run(p.String(), func(t *testing.T) {
+			t.Run("def df build", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, false, p, "build", "-F", ocisifPath, dockerfile)
+			})
+			t.Run("cstm df build", func(t *testing.T) {
+				c.dockerCredsPriorityTester(t, true, p, "build", "-F", ocisifPath, dockerfile)
+			})
+		})
+	}
+}
+
+func (c ctx) dockerCredsPriorityTester(t *testing.T, withCustomAuthFile bool, profile e2e.Profile, cmd string, args ...string) {
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-auth", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	prevCwd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("could not get current working directory: %s", err)
+	}
+	defer os.Chdir(prevCwd)
+	if err = os.Chdir(tmpdir); err != nil {
+		t.Fatalf("could not change cwd to %q: %s", tmpdir, err)
+	}
+
+	localAuthFileName := ""
+	if withCustomAuthFile {
+		localAuthFileName = "./my_local_authfile"
+	}
+
+	authFileArgs := []string{}
+	if withCustomAuthFile {
+		authFileArgs = []string{"--authfile", localAuthFileName}
+	}
+
+	// Store the previous values of relevant env vars, and set up facilities for
+	// wiping and restoring them.
+	envVarSet := []string{
+		"SINGULARITY_DOCKER_USERNAME",
+		"SINGULARITY_DOCKER_PASSWORD",
+		"DOCKER_USERNAME",
+		"DOCKER_PASSWORD",
+	}
+	prevEnvVals := make(map[string]string)
+	for _, varName := range envVarSet {
+		if varVal, ok := os.LookupEnv(varName); ok {
+			prevEnvVals[varName] = varVal
+		}
+	}
+	wipeVars := func() {
+		for _, varName := range envVarSet {
+			os.Unsetenv(varName)
+		}
+	}
+	restoreVars := func() {
+		wipeVars()
+		for _, varName := range envVarSet {
+			if varVal, ok := prevEnvVals[varName]; ok {
+				os.Setenv(varName, varVal)
+			}
+		}
+	}
+
+	t.Cleanup(func() {
+		e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+		restoreVars()
+	})
+
+	tests := []struct {
+		name            string
+		pfxDockerUser   string
+		pfxDockerPass   string
+		nopfxDockerUser string
+		nopfxDockerPass string
+		authLoggedIn    bool
+		expectExit      int
+	}{
+		{
+			name:          "pfx denv wrong, no auth",
+			pfxDockerUser: "wrong",
+			pfxDockerPass: "wrong",
+			authLoggedIn:  false,
+			expectExit:    255,
+		},
+		{
+			name:          "pfx denv wrong, auth",
+			pfxDockerUser: "wrong",
+			pfxDockerPass: "wrong",
+			authLoggedIn:  true,
+			expectExit:    255,
+		},
+		{
+			name:          "pfx denv right, no auth",
+			pfxDockerUser: e2e.DefaultUsername,
+			pfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:  false,
+			expectExit:    0,
+		},
+		{
+			name:          "pfx denv right, auth",
+			pfxDockerUser: e2e.DefaultUsername,
+			pfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:  true,
+			expectExit:    0,
+		},
+		{
+			name:            "nopfx denv wrong, no auth",
+			nopfxDockerUser: "wrong",
+			nopfxDockerPass: "wrong",
+			authLoggedIn:    false,
+			expectExit:      255,
+		},
+		{
+			name:            "nopfx denv wrong, auth",
+			nopfxDockerUser: "wrong",
+			nopfxDockerPass: "wrong",
+			authLoggedIn:    true,
+			expectExit:      255,
+		},
+		{
+			name:            "nopfx denv right, no auth",
+			nopfxDockerUser: e2e.DefaultUsername,
+			nopfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:    false,
+			expectExit:      0,
+		},
+		{
+			name:            "nopfx denv right, auth",
+			nopfxDockerUser: e2e.DefaultUsername,
+			nopfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:    true,
+			expectExit:      0,
+		},
+		{
+			name:            "both denv (pfx right), auth",
+			pfxDockerUser:   e2e.DefaultUsername,
+			pfxDockerPass:   e2e.DefaultPassword,
+			nopfxDockerUser: "wrong",
+			nopfxDockerPass: "wrong",
+			authLoggedIn:    true,
+			expectExit:      0,
+		},
+		{
+			name:            "both denv (pfx right), noauth",
+			pfxDockerUser:   e2e.DefaultUsername,
+			pfxDockerPass:   e2e.DefaultPassword,
+			nopfxDockerUser: "wrong",
+			nopfxDockerPass: "wrong",
+			authLoggedIn:    false,
+			expectExit:      0,
+		},
+		{
+			name:            "both denv (nopfx right), auth",
+			pfxDockerUser:   "wrong",
+			pfxDockerPass:   "wrong",
+			nopfxDockerUser: e2e.DefaultUsername,
+			nopfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:    true,
+			expectExit:      255,
+		},
+		{
+			name:            "both denv (nopfx right), noauth",
+			pfxDockerUser:   "wrong",
+			pfxDockerPass:   "wrong",
+			nopfxDockerUser: e2e.DefaultUsername,
+			nopfxDockerPass: e2e.DefaultPassword,
+			authLoggedIn:    false,
+			expectExit:      255,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			wipeVars()
+			if tt.pfxDockerUser != "" {
+				os.Setenv("SINGULARITY_DOCKER_USERNAME", tt.pfxDockerUser)
+			}
+			if tt.pfxDockerPass != "" {
+				os.Setenv("SINGULARITY_DOCKER_PASSWORD", tt.pfxDockerPass)
+			}
+			if tt.nopfxDockerUser != "" {
+				os.Setenv("DOCKER_USERNAME", tt.nopfxDockerUser)
+			}
+			if tt.nopfxDockerPass != "" {
+				os.Setenv("DOCKER_PASSWORD", tt.nopfxDockerPass)
+			}
+			if tt.authLoggedIn {
+				e2e.PrivateRepoLogin(t, c.env, profile, localAuthFileName)
+			} else {
+				e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+			}
+			c.env.RunSingularity(
+				t,
+				e2e.WithProfile(profile),
+				e2e.WithCommand(cmd),
+				e2e.WithArgs(append(authFileArgs, args...)...),
+				e2e.ExpectExit(tt.expectExit),
+			)
+		})
+	}
+}
+
+// AUFS whiteout tests
 func (c ctx) testDockerAUFS(t *testing.T) {
+	tests := []struct {
+		name       string
+		profile    e2e.Profile
+		keepLayers bool
+	}{
+		// Native SIF - whiteouts applied to squashed image via umoci rootfs
+		// extraction at creation.
+		{
+			name:       "NativeSIF",
+			profile:    e2e.UserProfile,
+			keepLayers: false,
+		},
+		// Single layer OCI-SIF - whiteouts applied to squashed image via
+		// oci-tools.Squash at creation.
+		{
+			name:       "OCISIF",
+			profile:    e2e.OCIUserProfile,
+			keepLayers: false,
+		},
+		// Multi layer OCI-SIF - whiteouts translated AUFS -> OverlayFS by
+		// oci-tools at creation and applied at runtime.
+		{
+			name:       "OCISIFKeepLayers",
+			profile:    e2e.OCIUserProfile,
+			keepLayers: true,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			c.aufsTest(t, tt.profile, tt.keepLayers)
+		})
+	}
+}
+
+func (c ctx) aufsTest(t *testing.T, profile e2e.Profile, keepLayers bool) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "aufs-", "")
 	t.Cleanup(func() {
 		if !t.Failed() {
@@ -276,11 +582,17 @@ func (c ctx) testDockerAUFS(t *testing.T) {
 	})
 	imagePath := filepath.Join(imageDir, "container")
 
+	args := []string{}
+	if keepLayers {
+		args = []string{"--keep-layers"}
+	}
+	args = append(args, imagePath, "docker://sylabsio/aufs-sanity")
+
 	c.env.RunSingularity(
 		t,
-		e2e.WithProfile(e2e.UserProfile),
-		e2e.WithCommand("build"),
-		e2e.WithArgs([]string{imagePath, "docker://sylabsio/aufs-sanity"}...),
+		e2e.WithProfile(profile),
+		e2e.WithCommand("pull"),
+		e2e.WithArgs(args...),
 		e2e.ExpectExit(0),
 	)
 
@@ -293,14 +605,22 @@ func (c ctx) testDockerAUFS(t *testing.T) {
 		argv []string
 		exit int
 	}{
+		// 'file2' should be present in three locations
 		{
 			name: "File 2",
 			argv: []string{imagePath, "ls", "/test/whiteout-dir/file2", "/test/whiteout-file/file2", "/test/normal-dir/file2"},
 			exit: 0,
 		},
+		// '/test/whiteout-file/file1' should be absent (via whiteout of the file)
 		{
-			name: "File1",
-			argv: []string{imagePath, "ls", "/test/whiteout-dir/file1", "/test/whiteout-file/file1"},
+			name: "WhiteoutFileFile1",
+			argv: []string{imagePath, "ls", "/test/whiteout-file/file1"},
+			exit: 1,
+		},
+		// '/test/whiteout-dir/file1' should be absent (via whiteout of the dir)
+		{
+			name: "WhiteoutDirFile1",
+			argv: []string{imagePath, "ls", "/test/whiteout-dir/file1"},
 			exit: 1,
 		},
 	}
@@ -309,7 +629,7 @@ func (c ctx) testDockerAUFS(t *testing.T) {
 		c.env.RunSingularity(
 			t,
 			e2e.AsSubtest(tt.name),
-			e2e.WithProfile(e2e.UserProfile),
+			e2e.WithProfile(profile),
 			e2e.WithCommand("exec"),
 			e2e.WithArgs(tt.argv...),
 			e2e.ExpectExit(tt.exit),
@@ -883,122 +1203,179 @@ func (c ctx) testDockerCMDQuotes(t *testing.T) {
 	)
 }
 
-// Check that the USER in a docker container is honored under --oci mode
-func (c ctx) testDockerUSER(t *testing.T) {
+// Check that the USER & WORKDIR in a docker container are honored under --oci mode
+func (c ctx) testDockerUSERWORKDIR(t *testing.T) {
 	dockerURI := "docker://sylabsio/docker-user"
+	dockerfile := filepath.Join("..", "test", "defs", "Dockerfile.customuser")
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, "", "dockerfile-build-USER-", "temp dir for OCI-SIF images")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+	userBuiltOCISIF := filepath.Join(tmpdir, "docker-user.oci.sif")
+	c.env.RunSingularity(
+		t,
+		e2e.AsSubtest("user df build"),
+		e2e.WithProfile(e2e.OCIUserProfile),
+		e2e.WithCommand("build"),
+		e2e.WithArgs(userBuiltOCISIF, dockerfile),
+		e2e.ExpectExit(0),
+	)
+	rootBuiltOCISIF := filepath.Join(tmpdir, "rootbuilt-docker-user.oci.sif")
+	c.env.RunSingularity(
+		t,
+		e2e.AsSubtest("root df build"),
+		e2e.WithProfile(e2e.OCIRootProfile),
+		e2e.WithCommand("build"),
+		e2e.WithArgs(rootBuiltOCISIF, dockerfile),
+		e2e.ExpectExit(0),
+	)
+
+	// Sanity check singularity native engine... no support for USER
+	c.env.RunSingularity(
+		t,
+		e2e.AsSubtest("default"),
+		e2e.WithProfile(e2e.UserProfile),
+		e2e.WithCommand("run"),
+		e2e.WithArgs(dockerURI),
+		e2e.ExpectExit(0, e2e.ExpectOutput(e2e.ContainMatch, fmt.Sprintf("uid=%d(%s) gid=%d",
+			e2e.UserProfile.ContainerUser(t).UID,
+			e2e.UserProfile.ContainerUser(t).Name,
+			e2e.UserProfile.ContainerUser(t).GID,
+		))),
+	)
+
+	metaTests := map[string]string{
+		"uri":          dockerURI,
+		"user oci-sif": userBuiltOCISIF,
+		"root oci-sif": rootBuiltOCISIF,
+	}
+
+	for subtestName, container := range metaTests {
+		t.Run(subtestName, func(t *testing.T) {
+			c.testDockerUSERWorker(t, container)
+		})
+	}
+}
+
+func (c ctx) testDockerUSERWorker(t *testing.T, container string) {
 	tests := []struct {
 		name          string
 		cmd           string
 		args          []string
+		wd            string
 		expectOutputs []e2e.SingularityCmdResultOp
-		profile       e2e.Profile
+		profiles      []e2e.Profile
 		expectExit    int
 	}{
-		// Sanity check singularity native engine... no support for USER
+		// `--oci` should honor container USER by default
 		{
-			name:    "default",
-			cmd:     "run",
-			profile: e2e.UserProfile,
-			args:    []string{dockerURI},
-			expectOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.ContainMatch, fmt.Sprintf("uid=%d(%s) gid=%d",
-					e2e.UserProfile.ContainerUser(t).UID,
-					e2e.UserProfile.ContainerUser(t).Name,
-					e2e.UserProfile.ContainerUser(t).GID,
-				)),
-			},
-		},
-		// `--oci` modes (USER honored by default)
-		{
-			name:    "OCIUser",
-			cmd:     "run",
-			profile: e2e.OCIUserProfile,
-			args:    []string{dockerURI},
+			name:     "OCIImageUser",
+			cmd:      "run",
+			profiles: []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile},
+			args:     []string{container},
 			expectOutputs: []e2e.SingularityCmdResultOp{
 				e2e.ExpectOutput(e2e.ContainMatch, `uid=2000(testuser) gid=2000(testgroup)`),
 			},
 		},
+		// `--fakeroot` is an explicit request for root in the container
 		{
-			name:    "OCIFakeroot",
-			profile: e2e.OCIFakerootProfile,
-			args:    []string{dockerURI},
+			name:     "OCIFakerootUser",
+			profiles: []e2e.Profile{e2e.OCIFakerootProfile},
+			args:     []string{container},
 			expectOutputs: []e2e.SingularityCmdResultOp{
 				e2e.ExpectOutput(e2e.ContainMatch, `uid=0(root) gid=0(root)`),
 			},
 		},
-		{
-			name:    "OCIRoot",
-			cmd:     "run",
-			profile: e2e.OCIRootProfile,
-			args:    []string{dockerURI},
-			expectOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.ContainMatch, `uid=2000(testuser) gid=2000(testgroup)`),
-			},
-		},
-		// `--oci` modes: check that we correctly error on conflict with `--home`
+
+		// At present, we don't support specifying `--home` when container declares a USER.
 		{
 			name:       "WithHomeOCIUser",
 			cmd:        "run",
-			profile:    e2e.OCIUserProfile,
-			args:       []string{"--home", "/tmp", dockerURI},
+			profiles:   []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile, e2e.OCIFakerootProfile},
+			args:       []string{"--home", "/tmp", container},
 			expectExit: 255,
 		},
+		// $HOME env var should match the container USER's home dir, by default.
 		{
-			name:       "WithHomeOCIFakeroot",
-			cmd:        "run",
-			profile:    e2e.OCIFakerootProfile,
-			args:       []string{"--home", "/tmp", dockerURI},
-			expectExit: 255,
-		},
-		{
-			name:       "WithHomeOCIRoot",
-			cmd:        "run",
-			profile:    e2e.OCIRootProfile,
-			args:       []string{"--home", "/tmp", dockerURI},
-			expectExit: 255,
-		},
-		// `--oci` modes: check that we don't override container-user's home directory
-		{
-			name:    "OrigHomeOCIUser",
-			cmd:     "exec",
-			profile: e2e.OCIUserProfile,
-			args:    []string{dockerURI, "env"},
+			name:     "OCIImageHomeEnv",
+			cmd:      "exec",
+			profiles: []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile},
+			args:     []string{container, "env"},
 			expectOutputs: []e2e.SingularityCmdResultOp{
 				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/home/testuser\b`),
 			},
 			expectExit: 0,
 		},
+		// `--fakeroot` is an explicit request for root in the container, so verify home dir.
 		{
-			name:    "OrigHomeOCIFakeroot",
-			cmd:     "exec",
-			profile: e2e.OCIFakerootProfile,
-			args:    []string{dockerURI, "env"},
+			name:     "OCIFakerootHomeEnv",
+			cmd:      "exec",
+			profiles: []e2e.Profile{e2e.OCIFakerootProfile},
+			args:     []string{container, "env"},
 			expectOutputs: []e2e.SingularityCmdResultOp{
 				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/root\b`),
 			},
 			expectExit: 0,
 		},
+		// USER's home directory should always be owned by USER
 		{
-			name:    "OrigHomeOCIRoot",
-			cmd:     "exec",
-			profile: e2e.OCIRootProfile,
-			args:    []string{dockerURI, "env"},
+			name:     "OCIImageHomePerms",
+			cmd:      "exec",
+			profiles: []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile, e2e.OCIFakerootProfile},
+			args:     []string{container, "stat", "-c", "%U(%u):%G(%g)", "/home/testuser"},
 			expectOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/home/testuser\b`),
+				e2e.ExpectOutput(e2e.ExactMatch, "testuser(2000):testgroup(2000)"),
+			},
+			expectExit: 0,
+		},
+		// WORKDIR should be honored, by default.
+		{
+			name:     "OCIImageWorkdir",
+			cmd:      "exec",
+			profiles: []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile, e2e.OCIFakerootProfile},
+			args:     []string{container, "pwd"},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ExactMatch, "/home/testuser"),
+			},
+			expectExit: 0,
+		},
+		// --no-compat emulates native mode, so WORKDIR is ignored and container is entered at host CWD.
+		{
+			name:     "OCINoCompatWorkdir",
+			cmd:      "exec",
+			profiles: []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile, e2e.OCIFakerootProfile},
+			args:     []string{"--no-compat", container, "pwd"},
+			wd:       "/tmp",
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ExactMatch, "/tmp"),
 			},
 			expectExit: 0,
 		},
 	}
 
 	for _, tt := range tests {
-		c.env.RunSingularity(
-			t,
-			e2e.AsSubtest(tt.name),
-			e2e.WithProfile(tt.profile),
-			e2e.WithCommand("run"),
-			e2e.WithArgs(tt.args...),
-			e2e.ExpectExit(tt.expectExit, tt.expectOutputs...),
-		)
+		for _, profile := range tt.profiles {
+			cmd := "run"
+			if tt.cmd != "" {
+				cmd = tt.cmd
+			}
+			cmdOps := []e2e.SingularityCmdOp{
+				e2e.AsSubtest(tt.name + "/" + profile.String()),
+				e2e.WithProfile(profile),
+				e2e.WithCommand(cmd),
+				e2e.WithArgs(tt.args...),
+				e2e.ExpectExit(tt.expectExit, tt.expectOutputs...),
+			}
+			if tt.wd != "" {
+				cmdOps = append(cmdOps, e2e.WithDir(tt.wd))
+			}
+			c.env.RunSingularity(
+				t,
+				cmdOps...,
+			)
+		}
 	}
 }
 
@@ -1141,12 +1518,245 @@ func checkNativeSIFPlatform(t *testing.T, imgPath, platform string) {
 	}
 }
 
+// Test that we can perform cross-architecture builds from Dockerfile using buildkit
+func (c ctx) testDockerCrossArchBk(t *testing.T) {
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, "", "dockerfile_crossarch_", "dir")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	dockerfile, err := e2e.WriteTempFile(tmpdir, "Dockerfile", `
+FROM alpine
+CMD /bin/true
+`)
+	if err != nil {
+		t.Fatalf("While trying to create temporary Dockerfile: %v", err)
+	}
+
+	arch := getNonNativeArch()
+	profiles := []e2e.Profile{e2e.OCIUserProfile, e2e.OCIRootProfile}
+	for _, profile := range profiles {
+		imgPath := filepath.Join(tmpdir, "image."+profile.String()+".oci.sif")
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(profile.String()),
+			e2e.WithProfile(profile),
+			e2e.WithCommand("build"),
+			e2e.WithArgs("--arch", arch, imgPath, dockerfile),
+			e2e.ExpectExit(0),
+			e2e.PostRun(func(t *testing.T) {
+				verifyImgArch(t, imgPath, arch)
+			}),
+		)
+	}
+}
+
+func getNonNativeArch() string {
+	nativeArch := runtime.GOARCH
+	switch nativeArch {
+	case "amd64":
+		return "arm64"
+	default:
+		return "amd64"
+	}
+}
+
+func verifyImgArch(t *testing.T, imgPath, arch string) {
+	fi, err := sif.LoadContainerFromPath(imgPath, sif.OptLoadWithFlag(os.O_RDONLY))
+	if err != nil {
+		t.Fatalf("while loading SIF (%s): %v", imgPath, err)
+	}
+	defer fi.UnloadContainer()
+
+	ix, err := ocisif.ImageIndexFromFileImage(fi)
+	if err != nil {
+		t.Fatalf("while obtaining image index from %s: %v", imgPath, err)
+	}
+	idxManifest, err := ix.IndexManifest()
+	if err != nil {
+		t.Fatalf("while obtaining index manifest from %s: %v", imgPath, err)
+	}
+	if len(idxManifest.Manifests) != 1 {
+		t.Fatalf("while reading %s: single manifest expected, found %d manifests", imgPath, len(idxManifest.Manifests))
+	}
+	imageDigest := idxManifest.Manifests[0].Digest
+
+	img, err := ix.Image(imageDigest)
+	if err != nil {
+		t.Fatalf("while initializing image from %s: %v", imgPath, err)
+	}
+
+	cg, err := img.ConfigFile()
+	if err != nil {
+		t.Fatalf("while accessing config for %s: %v", imgPath, err)
+	}
+
+	assert.Equal(t, arch, cg.Architecture)
+}
+
+// Test support for SCIF containers in OCI mode
+func (c ctx) testDockerSCIF(t *testing.T) {
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, "", "docker-scif-", "dir")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	scifRecipeFilename := "local_scif_recipe"
+	scifRecipeFullpath := filepath.Join(tmpdir, scifRecipeFilename)
+	scifRecipeSource := filepath.Join("..", "test", "defs", "scif_recipe")
+	if err := fs.CopyFile(scifRecipeSource, scifRecipeFullpath, 0o755); err != nil {
+		t.Fatalf("While trying to copy %q to %q: %v", scifRecipeSource, scifRecipeFullpath, err)
+	}
+
+	tmplValues := struct{ SCIFRecipeFilename string }{SCIFRecipeFilename: scifRecipeFilename}
+	scifDockerfile := tmpl.Execute(t, tmpdir, "Dockerfile-", filepath.Join("..", "test", "defs", "Dockerfile.scif.tmpl"), tmplValues)
+	scifImageFilename := "scif-image.oci.sif"
+	scifImageFullpath := filepath.Join(tmpdir, scifImageFilename)
+
+	// Uncomment when `singularity inspect --oci` for Docker-style SCIF
+	// containers is enabled.
+	// See: https://github.com/sylabs/singularity/pull/2360
+	// scifInspectOutAllPath := filepath.Join("..", "test", "defs", "scif_recipe.inspect_output.all")
+	// scifInspectOutAllBytes, err := os.ReadFile(scifInspectOutAllPath)
+	// if err != nil {
+	// 	t.Fatalf("While trying to read contents of %s: %v", scifInspectOutAllPath, err)
+	// }
+	// scifInspectOutOnePath := filepath.Join("..", "test", "defs", "scif_recipe.inspect_output.one")
+	// scifInspectOutOneBytes, err := os.ReadFile(scifInspectOutOnePath)
+	// if err != nil {
+	// 	t.Fatalf("While trying to read contents of %s: %v", scifInspectOutOnePath, err)
+	// }
+
+	// testInspectOutput := func(bytes []byte) func(t *testing.T, r *e2e.SingularityCmdResult) {
+	// 	return func(t *testing.T, r *e2e.SingularityCmdResult) {
+	// 		got := string(r.Stdout)
+	// 		assert.Equal(t, got, string(bytes))
+	// 	}
+	// }
+
+	c.env.RunSingularity(
+		t,
+		e2e.AsSubtest("build"),
+		e2e.WithProfile(e2e.OCIUserProfile),
+		e2e.WithCommand("build"),
+		e2e.WithDir(tmpdir),
+		e2e.WithArgs(scifImageFilename, scifDockerfile),
+		e2e.ExpectExit(0),
+	)
+
+	tests := []struct {
+		name       string
+		cmd        string
+		app        string
+		preArgs    []string
+		args       []string
+		expects    []e2e.SingularityCmdResultOp
+		expectExit int
+	}{
+		{
+			name: "run echo",
+			cmd:  "run",
+			app:  "hello-world-echo",
+			expects: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, "The best app is hello-world-echo"),
+			},
+			expectExit: 0,
+		},
+		{
+			name: "exec echo",
+			cmd:  "exec",
+			app:  "hello-world-echo",
+			args: []string{"echo", "This is different text that should still include [e]SCIF_APPNAME"},
+			expects: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, "This is different text that should still include hello-world-echo"),
+			},
+			expectExit: 0,
+		},
+		{
+			name: "run script",
+			cmd:  "run",
+			app:  "hello-world-script",
+			expects: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, "Hello World!"),
+			},
+			expectExit: 0,
+		},
+		{
+			name: "exec script",
+			cmd:  "exec",
+			app:  "hello-world-script",
+			args: []string{"echo", "This is different text that should still include [e]SCIF_APPNAME"},
+			expects: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, "This is different text that should still include hello-world-script"),
+			},
+			expectExit: 0,
+		},
+		{
+			name: "exec script2",
+			cmd:  "exec",
+			app:  "hello-world-script",
+			args: []string{"/bin/bash hello-world.sh"},
+			expects: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, "Hello World!"),
+			},
+			expectExit: 0,
+		},
+		// Uncomment when `singularity inspect --oci` for Docker-style SCIF
+		// containers is enabled.
+		// See: https://github.com/sylabs/singularity/pull/2360
+		// {
+		//  name:    "insp all",
+		//  cmd:     "inspect",
+		//  preArgs: []string{"--oci", "--list-apps"},
+		//  expects: []e2e.SingularityCmdResultOp{
+		//      testInspectOutput(scifInspectOutAllBytes),
+		//  },
+		//  expectExit: 0,
+		// }, {
+		//  name:    "insp one",
+		//  cmd:     "inspect",
+		//  app:     "hello-world-script",
+		//  preArgs: []string{"--oci"},
+		//  expects: []e2e.SingularityCmdResultOp{
+		//      testInspectOutput(scifInspectOutOneBytes),
+		//  },
+		//  expectExit: 0,
+		// },
+	}
+
+	for _, tt := range tests {
+		args := tt.preArgs[:]
+		if tt.app != "" {
+			args = append(args, "--app", tt.app)
+		}
+		args = append(args, scifImageFullpath)
+		if len(tt.args) > 0 {
+			args = append(args, tt.args...)
+		}
+
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(e2e.OCIUserProfile),
+			e2e.WithCommand(tt.cmd),
+			e2e.WithArgs(args...),
+			e2e.ExpectExit(0, tt.expects...),
+		)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := ctx{
 		env: env,
 	}
 
+	np := testhelper.NoParallel
+
 	return testhelper.Tests{
 		// Run most docker:// source tests sequentially amongst themselves, so we
 		// don't hit DockerHub massively in parallel, and we benefit from
@@ -1162,8 +1772,10 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 			t.Run("entrypoint", c.testDockerENTRYPOINT)
 			t.Run("cmdentrypoint", c.testDockerCMDENTRYPOINT)
 			t.Run("cmd quotes", c.testDockerCMDQuotes)
-			t.Run("user", c.testDockerUSER)
+			t.Run("user workdir", c.testDockerUSERWORKDIR)
 			t.Run("platform", c.testDockerPlatform)
+			t.Run("crossarch buildkit", c.testDockerCrossArchBk)
+			t.Run("scif", c.testDockerSCIF)
 			// Regressions
 			t.Run("issue 4524", c.issue4524)
 			t.Run("issue 1286", c.issue1286)
@@ -1175,6 +1787,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		// registry, can be run in parallel, with `--disable-cache` used within
 		// them to avoid docker caching concurrency issues.
 		"docker host": c.testDockerHost,
+		"cred prio":   np(c.testDockerCredsPriority),
 		"registry":    c.testDockerRegistry,
 		// Regressions
 		"issue 4943": c.issue4943,
