@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -9,7 +9,6 @@
 package pull
 
 import (
-	"context"
 	"fmt"
 	"net/http"
 	"net/http/httptest"
@@ -18,15 +17,19 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/containerd/containerd/reference"
-	"github.com/containerd/containerd/remotes/docker"
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	syoras "github.com/sylabs/singularity/internal/pkg/client/oras"
-	"github.com/sylabs/singularity/internal/pkg/util/uri"
+	"github.com/google/go-containerregistry/pkg/name"
+	"github.com/google/go-containerregistry/pkg/v1/remote"
+	"github.com/google/go-containerregistry/pkg/v1/types"
+	ocisif "github.com/sylabs/oci-tools/pkg/sif"
+	"github.com/sylabs/sif/v2/pkg/sif"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/oras"
+	syoras "github.com/sylabs/singularity/v4/internal/pkg/client/oras"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/uri"
+	"github.com/sylabs/singularity/v4/pkg/image"
 	"golang.org/x/sys/unix"
-	"oras.land/oras-go/pkg/content"
-	"oras.land/oras-go/pkg/oras"
 )
 
 type ctx struct {
@@ -34,24 +37,35 @@ type ctx struct {
 }
 
 type testStruct struct {
-	desc             string // case description
-	srcURI           string // source URI for image
-	library          string // use specific library, XXX(mem): not tested yet
-	arch             string // architecture to force, if any
-	force            bool   // pass --force
-	createDst        bool   // create destination file before pull
-	unauthenticated  bool   // pass --allow-unauthenticated
-	setImagePath     bool   // pass destination path
-	setPullDir       bool   // pass --dir
-	expectedExitCode int
-	workDir          string
-	pullDir          string
-	imagePath        string
-	expectedImage    string
-	envVars          []string
+	desc              string // case description
+	srcURI            string // source URI for image
+	library           string // use specific library server URI
+	arch              string // architecture to force, if any
+	platform          string // platform to force, if any
+	force             bool   // pass --force
+	createDst         bool   // create destination file before pull
+	unauthenticated   bool   // pass --allow-unauthenticated
+	setImagePath      bool   // pass destination path (singularity pull <image path> <source>)
+	setPullDir        bool   // pass --dir
+	oci               bool   // pass --oci
+	keepLayers        bool   // pass --keep-layers
+	noHTTPS           bool   // pass --no-https
+	expectedExitCode  int
+	workDir           string
+	pullDir           string
+	imagePath         string
+	expectedImage     string
+	expectedOCI       bool
+	expectedOCILayers int // number of squashfs layers if OCI-SIF
+	envVars           []string
+	requirements      func(t *testing.T)
 }
 
 func (c *ctx) imagePull(t *testing.T, tt testStruct) {
+	if tt.requirements != nil {
+		tt.requirements(t)
+	}
+
 	// Use a one-time cache directory specific to this pull. This ensures we are always
 	// testing an entire pull operation, performing the download into an empty cache.
 	cacheDir, cleanup := e2e.MakeCacheDir(t, "")
@@ -71,6 +85,10 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 		argv += "--arch " + tt.arch + " "
 	}
 
+	if tt.platform != "" {
+		argv += "--platform " + tt.platform + " "
+	}
+
 	if tt.force {
 		argv += "--force "
 	}
@@ -87,6 +105,18 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 		argv += "--library " + tt.library + " "
 	}
 
+	if tt.oci {
+		argv += "--oci "
+	}
+
+	if tt.keepLayers {
+		argv += "--keep-layers "
+	}
+
+	if tt.noHTTPS {
+		argv += "--no-https "
+	}
+
 	if tt.imagePath != "" {
 		argv += tt.imagePath + " "
 	}
@@ -103,7 +133,6 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 
 	c.env.RunSingularity(
 		t,
-		e2e.AsSubtest(tt.desc),
 		e2e.WithProfile(e2e.UserProfile),
 		e2e.WithEnv(tt.envVars),
 		e2e.WithDir(tt.workDir),
@@ -114,7 +143,7 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 	checkPullResult(t, tt)
 }
 
-func getImageNameFromURI(imgURI string) string {
+func getImageNameFromURI(imgURI string, oci bool) string {
 	// XXX(mem): this function should be part of the code, not the test
 	switch transport, ref := uri.Split(imgURI); {
 	case ref == "":
@@ -124,19 +153,21 @@ func getImageNameFromURI(imgURI string) string {
 		imgURI = "library://" + imgURI
 	}
 
-	return uri.GetName(imgURI)
+	suffix := "sif"
+	if oci {
+		suffix = "oci.sif"
+	}
+
+	return uri.Filename(imgURI, suffix)
 }
 
 func (c *ctx) setup(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
+	e2e.EnsureOCISIF(t, c.env)
+	e2e.EnsureORASOCISIF(t, c.env)
+	e2e.EnsureRegistryOCISIF(t, c.env)
 
-	// setup file and dir to use as invalid images
-	orasInvalidDir, err := os.MkdirTemp(c.env.TestDir, "oras_push_dir-")
-	if err != nil {
-		t.Fatalf("unable to create src dir for push tests: %v", err)
-	}
-
-	orasInvalidFile, err := e2e.WriteTempFile(orasInvalidDir, "oras_invalid_image-", "Invalid Image Contents")
+	orasInvalidFile, err := e2e.WriteTempFile(c.env.TestDir, "oras_invalid_image-", "Invalid Image Contents")
 	if err != nil {
 		t.Fatalf("unable to create src file for push tests: %v", err)
 	}
@@ -160,13 +191,13 @@ func (c *ctx) setup(t *testing.T) {
 			layerMediaType: syoras.SifLayerMediaTypeProto,
 		},
 		{
-			srcPath:        orasInvalidDir,
-			uri:            fmt.Sprintf("%s/pull_test_dir:latest", c.env.TestRegistry),
+			srcPath:        orasInvalidFile,
+			uri:            fmt.Sprintf("%s/pull_test_invalid_file:latest", c.env.TestRegistry),
 			layerMediaType: syoras.SifLayerMediaTypeV1,
 		},
 		{
-			srcPath:        orasInvalidFile,
-			uri:            fmt.Sprintf("%s/pull_test_invalid_file:latest", c.env.TestRegistry),
+			srcPath:        c.env.OCISIFPath,
+			uri:            fmt.Sprintf("%s/pull_test_oci-sif:latest", c.env.TestRegistry),
 			layerMediaType: syoras.SifLayerMediaTypeV1,
 		},
 	}
@@ -179,25 +210,28 @@ func (c *ctx) setup(t *testing.T) {
 	}
 }
 
+//nolint:maintidx
 func (c ctx) testPullCmd(t *testing.T) {
 	tests := []testStruct{
+		//
+		// library:// URIs
+		// SCS / Singularity Enterprise & compatible.
+		//
 		{
-			desc:             "non existent image",
+			desc:             "library non-existent",
 			srcURI:           "library://sylabs/tests/does_not_exist:0",
 			expectedExitCode: 255,
 		},
-
 		// --allow-unauthenticated tests
 		{
-			desc:             "unsigned image allow unauthenticated",
+			desc:             "library allow-unauthenticated",
 			srcURI:           "library://sylabs/tests/unsigned:1.0.0",
 			unauthenticated:  true,
 			expectedExitCode: 0,
 		},
-
 		// --force tests
 		{
-			desc:             "force existing file",
+			desc:             "library force existing",
 			srcURI:           "library://alpine:3.11.5",
 			force:            true,
 			createDst:        true,
@@ -205,7 +239,7 @@ func (c ctx) testPullCmd(t *testing.T) {
 			expectedExitCode: 0,
 		},
 		{
-			desc:             "force non-existing file",
+			desc:             "library force non-existing",
 			srcURI:           "library://alpine:3.11.5",
 			force:            true,
 			createDst:        false,
@@ -214,31 +248,42 @@ func (c ctx) testPullCmd(t *testing.T) {
 		},
 		{
 			// --force should not have an effect on --allow-unauthenticated=false
-			desc:             "unsigned image force require authenticated",
+			desc:             "library force allow-unauthenticated",
 			srcURI:           "library://sylabs/tests/unsigned:1.0.0",
 			force:            true,
 			unauthenticated:  false,
 			expectedExitCode: 0,
 		},
-
 		// test version specifications
 		{
-			desc:             "image with specific hash",
+			desc:             "library hash",
 			srcURI:           "library://alpine:sha256.03883ca565b32e58fa0a496316d69de35741f2ef34b5b4658a6fec04ed8149a8",
 			arch:             "amd64",
 			unauthenticated:  true,
 			expectedExitCode: 0,
 		},
 		{
-			desc:             "latest tag",
+			desc:             "library tag",
 			srcURI:           "library://alpine:latest",
 			unauthenticated:  true,
 			expectedExitCode: 0,
 		},
-
+		// forced arch and platform equivalent
+		{
+			desc:             "library non-oci arch",
+			srcURI:           "library://alpine:3.11.5",
+			arch:             "ppc64le",
+			expectedExitCode: 0,
+		},
+		{
+			desc:             "library non-oci platform",
+			srcURI:           "library://alpine:3.11.5",
+			platform:         "linux/ppc64le",
+			expectedExitCode: 0,
+		},
 		// --dir tests
 		{
-			desc:             "dir no image path",
+			desc:             "library dir",
 			srcURI:           "library://alpine:3.11.5",
 			unauthenticated:  true,
 			setPullDir:       true,
@@ -253,76 +298,35 @@ func (c ctx) testPullCmd(t *testing.T) {
 			// /tmp/a/b/image.sif, the code expects to find /tmp/a/b/c/tmp/a/b/image.sif. Since
 			// the directory /tmp/a/b/c/tmp/a/b does not exist, it fails to create the file
 			// image.sif in there.
-			desc:             "dir image path",
+			desc:             "library dir with image path",
 			srcURI:           "library://alpine:3.11.5",
 			unauthenticated:  true,
 			setPullDir:       true,
 			setImagePath:     true,
 			expectedExitCode: 255,
 		},
-
-		// transport tests
+		// default transport should be library
 		{
-			desc:             "bare image name",
+			desc:             "library default transport",
 			srcURI:           "alpine:3.11.5",
 			force:            true,
 			unauthenticated:  true,
 			expectedExitCode: 0,
 		},
-		// TODO(mem): reenable this; disabled while shub is down
-		// {
-		// 	desc:            "image from shub",
-		// 	srcURI:          "shub://GodloveD/busybox",
-		// 	force:           true,
-		// 	unauthenticated: false,
-		// 	expectSuccess:   true,
-		// },
-		// Finalized v1 layer mediaType (3.7 and onward)
-		{
-			desc:             "oras transport for SIF from registry",
-			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_sif:latest", // TODO(mem): obtain registry from context
-			force:            true,
-			unauthenticated:  false,
-			expectedExitCode: 0,
-		},
-		// Original/prototype layer mediaType (<3.7)
-		{
-			desc:             "oras transport for SIF from registry (SifLayerMediaTypeProto)",
-			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_sif_mediatypeproto:latest", // TODO(mem): obtain registry from context
-			force:            true,
-			unauthenticated:  false,
-			expectedExitCode: 0,
-		},
-
-		// pulling of invalid images with oras
-		{
-			desc:             "oras pull of non SIF file",
-			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_:latest", // TODO(mem): obtain registry from context
-			force:            true,
-			expectedExitCode: 255,
-		},
-		{
-			desc:             "oras pull of packed dir",
-			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_invalid_file:latest", // TODO(mem): obtain registry from context
-			force:            true,
-			expectedExitCode: 255,
-		},
-
 		// pulling with library URI argument
 		{
-			desc:             "bad library URI",
+			desc:             "library bad library flag",
 			srcURI:           "library://busybox:1.31.1",
 			library:          "https://bad-library.sylabs.io",
 			expectedExitCode: 255,
 		},
 		{
-			desc:             "default library URI",
+			desc:             "library default library flag",
 			srcURI:           "library://busybox:1.31.1",
 			library:          "https://library.sylabs.io",
 			force:            true,
 			expectedExitCode: 0,
 		},
-
 		// pulling with library URI containing host name and library argument
 		{
 			desc:             "library URI containing host name and library argument",
@@ -330,7 +334,6 @@ func (c ctx) testPullCmd(t *testing.T) {
 			library:          "https://notlibrary.sylabs.io",
 			expectedExitCode: 255,
 		},
-
 		// pulling with library URI containing host name
 		{
 			desc:             "library URI containing bad host name",
@@ -343,6 +346,166 @@ func (c ctx) testPullCmd(t *testing.T) {
 			force:            true,
 			expectedExitCode: 0,
 		},
+		// pulling an OCI-SIF image from library backing registry
+		{
+			desc:   "library oci-sif fallback",
+			srcURI: "library://sylabs/tests/alpine-oci-sif:latest",
+			// will try library protocol first, should then attempt oci pull
+			oci:               false,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			expectedExitCode:  0,
+			requirements: func(t *testing.T) {
+				require.Arch(t, "amd64")
+			},
+		},
+		{
+			desc:   "library oci-sif direct",
+			srcURI: "library://sylabs/tests/alpine-oci-sif:latest",
+			// direct oci pull
+			oci:               true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			expectedExitCode:  0,
+			requirements: func(t *testing.T) {
+				require.Arch(t, "amd64")
+			},
+		},
+		{
+			desc:   "library oci-sif platform",
+			srcURI: "library://sylabs/tests/alpine-ppc64le-oci-sif:latest",
+			// direct oci pull
+			platform:          "linux/ppc64le",
+			oci:               true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			expectedExitCode:  0,
+		},
+		{
+			desc:   "library oci-sif arch",
+			srcURI: "library://sylabs/tests/alpine-ppc64le-oci-sif:latest",
+			// direct oci pull
+			arch:              "ppc64le",
+			oci:               true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			expectedExitCode:  0,
+		},
+		//
+		// shub:// URIs
+		// Singularity Hub (retired) and compatible.
+		//
+		// TODO(mem): reenable this; disabled while shub is down
+		// {
+		// 	desc:            "image from shub",
+		// 	srcURI:          "shub://GodloveD/busybox",
+		// 	force:           true,
+		// 	unauthenticated: false,
+		// 	expectSuccess:   true,
+		// },
+
+		//
+		// oras:// URIs
+		// SIF file as ORAS / OCI artifact.
+		//
+		// Finalized v1 layer mediaType (3.7 and onward)
+		{
+			desc:             "oras transport for SIF from registry",
+			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_sif:latest",
+			force:            true,
+			unauthenticated:  false,
+			expectedExitCode: 0,
+		},
+		// Original/prototype layer mediaType (<3.7)
+		{
+			desc:             "oras transport for SIF from registry (SifLayerMediaTypeProto)",
+			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_sif_mediatypeproto:latest",
+			force:            true,
+			unauthenticated:  false,
+			expectedExitCode: 0,
+		},
+		// OCI-SIF
+		{
+			desc:              "oras pull of oci-sif",
+			srcURI:            "oras://" + c.env.TestRegistry + "/pull_test_oci-sif:latest",
+			force:             true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			expectedExitCode:  0,
+		},
+		// Invalid (non-SIF) artifacts
+		{
+			desc:             "oras pull of non SIF file",
+			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_:latest",
+			force:            true,
+			expectedExitCode: 255,
+		},
+		{
+			desc:             "oras pull of packed dir",
+			srcURI:           "oras://" + c.env.TestRegistry + "/pull_test_invalid_file:latest",
+			force:            true,
+			expectedExitCode: 255,
+		},
+
+		//
+		// docker:// URIs
+		// Standard OCI images, and OCI-SIF single layer squashfs images, in an OCI distribution-spec registry.
+		//
+		// pulling a standard OCI image from local registry to a native SIF
+		{
+			desc:              "docker oci to sif",
+			srcURI:            c.env.TestRegistryImage,
+			oci:               false,
+			expectedOCI:       false,
+			expectedOCILayers: 1,
+			noHTTPS:           true,
+			force:             true,
+			expectedExitCode:  0,
+		},
+		// pulling a standard OCI image from local registry to an OCI-SIF
+		{
+			desc:              "docker oci to oci-sif",
+			srcURI:            c.env.TestRegistryImage,
+			oci:               true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			noHTTPS:           true,
+			force:             true,
+			expectedExitCode:  0,
+		},
+		// pulling an OCI-SIF image from local registry to an OCI-SIF
+		{
+			desc:              "docker oci-sif to oci-sif",
+			srcURI:            c.env.TestRegistryOCISIF,
+			oci:               true,
+			expectedOCI:       true,
+			expectedOCILayers: 1,
+			noHTTPS:           true,
+			force:             true,
+			expectedExitCode:  0,
+		},
+		// pulling an OCI-SIF image from local registry to a native SIF (not implemented)
+		{
+			desc:             "docker oci-sif to sif",
+			srcURI:           c.env.TestRegistryOCISIF,
+			oci:              false,
+			expectedOCI:      false,
+			noHTTPS:          true,
+			force:            true,
+			expectedExitCode: 255,
+		},
+		// pulling an OCI-SIF image from local registry to a multi-layer OCI-SIF
+		{
+			desc:              "docker oci-sif to multi-layer oci-sif",
+			srcURI:            c.env.TestRegistryLayeredImage,
+			oci:               true,
+			keepLayers:        true,
+			expectedOCI:       true,
+			expectedOCILayers: 8,
+			noHTTPS:           true,
+			force:             true,
+			expectedExitCode:  0,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
@@ -370,14 +533,13 @@ func (c ctx) testPullCmd(t *testing.T) {
 				// No explicit image path specified. Will use temp dir as working directory,
 				// so we pull into a clean location.
 				tt.workDir = tmpdir
-				imageName := getImageNameFromURI(tt.srcURI)
+				imageName := getImageNameFromURI(tt.srcURI, tt.oci)
 				tt.expectedImage = filepath.Join(tmpdir, imageName)
 
 				// if there's a pullDir, that's where we expect to find the image
 				if tt.pullDir != "" {
 					tt.expectedImage = filepath.Join(tt.pullDir, imageName)
 				}
-
 			}
 
 			// In order to actually test force, there must already be a file present in
@@ -396,28 +558,92 @@ func (c ctx) testPullCmd(t *testing.T) {
 }
 
 func checkPullResult(t *testing.T, tt testStruct) {
-	if tt.expectedExitCode == 0 {
-		_, err := os.Stat(tt.expectedImage)
-		switch err {
-		case nil:
-			// PASS
-			return
+	if tt.expectedExitCode != 0 {
+		return
+	}
+	_, err := os.Stat(tt.expectedImage)
+	switch err {
+	case nil:
+		// PASS
 
-		case os.ErrNotExist:
-			// FAIL
-			t.Errorf("expecting image at %q, not found: %+v\n", tt.expectedImage, err)
+	case os.ErrNotExist:
+		// FAIL
+		t.Errorf("expecting image at %q, not found: %+v\n", tt.expectedImage, err)
 
-		default:
-			// FAIL
-			t.Errorf("unable to stat image at %q: %+v\n", tt.expectedImage, err)
+	default:
+		// FAIL
+		t.Errorf("unable to stat image at %q: %+v\n", tt.expectedImage, err)
+	}
+
+	// image.Init does an architecture check... so we can't call it if we are
+	// pulling a foreign arch image on purpose.
+	if tt.arch != "" || tt.platform != "" {
+		return
+	}
+
+	img, err := image.Init(tt.expectedImage, false)
+	if err != nil {
+		t.Fatalf("while checking image: %v", err)
+	}
+	defer img.File.Close()
+
+	switch img.Type {
+	case image.SIF:
+		if tt.expectedOCI {
+			t.Errorf("Native SIF pulled, but --oci specified")
 		}
+	case image.OCISIF:
+		if !tt.expectedOCI {
+			t.Errorf("OCI-SIF pulled, but --oci not specified")
+		}
+		checkOCISIF(t, tt.expectedImage, tt.expectedOCILayers)
+	default:
+		t.Errorf("Unexpected image type %d", img.Type)
+	}
+}
 
-		// XXX(mem): This is running a bunch of commands in the downloaded
-		// images. Do we really want this here? If yes, we need to have a
-		// way to do this in a generic fashion, as it's going to be shared
-		// with build as well.
+func checkOCISIF(t *testing.T, imgFile string, expectLayers int) {
+	fi, err := sif.LoadContainerFromPath(imgFile, sif.OptLoadWithFlag(os.O_RDONLY))
+	if err != nil {
+		t.Fatalf("while loading SIF: %v", err)
+	}
+	defer fi.UnloadContainer()
 
-		// imageVerify(t, tt.imagePath, false)
+	ix, err := ocisif.ImageIndexFromFileImage(fi)
+	if err != nil {
+		t.Fatalf("while obtaining image index: %v", err)
+	}
+	idxManifest, err := ix.IndexManifest()
+	if err != nil {
+		t.Fatalf("while obtaining index manifest: %v", err)
+	}
+	if len(idxManifest.Manifests) != 1 {
+		t.Fatalf("single manifest expected, found %d manifests", len(idxManifest.Manifests))
+	}
+	imageDigest := idxManifest.Manifests[0].Digest
+
+	img, err := ix.Image(imageDigest)
+	if err != nil {
+		t.Fatalf("while initializing image: %v", err)
+	}
+
+	layers, err := img.Layers()
+	if err != nil {
+		t.Fatalf("while fetching image layers: %v", err)
+	}
+
+	if len(layers) != expectLayers {
+		t.Errorf("expected %d layers, found %d", expectLayers, len(layers))
+	}
+
+	for i, l := range layers {
+		mt, err := l.MediaType()
+		if err != nil {
+			t.Fatalf("while examining layer: %v", err)
+		}
+		if mt != "application/vnd.sylabs.image.layer.v1.squashfs" {
+			t.Errorf("layer %d: unsupported layer mediaType %q", i, mt)
+		}
 	}
 }
 
@@ -427,56 +653,24 @@ func checkPullResult(t *testing.T, tt testStruct) {
 // We can also set the layer mediaType - so we can push images with older media types
 // to verify that they can still be pulled.
 func orasPushNoCheck(path, ref, layerMediaType string) error {
+	ref = strings.TrimPrefix(ref, "oras://")
 	ref = strings.TrimPrefix(ref, "//")
 
-	spec, err := reference.Parse(ref)
+	// Get reference to image in the remote
+	ir, err := name.ParseReference(ref,
+		name.WithDefaultTag(name.DefaultTag),
+		name.WithDefaultRegistry(name.DefaultRegistry),
+	)
 	if err != nil {
-		return fmt.Errorf("unable to parse oci reference: %w", err)
+		return err
 	}
 
-	// Hostname() will panic if there is no '/' in the locator
-	// explicitly check for this and fail in order to prevent panic
-	// this case will only occur for incorrect uris
-	if !strings.Contains(spec.Locator, "/") {
-		return fmt.Errorf("not a valid oci object uri: %s", ref)
-	}
-
-	// append default tag if no object exists
-	if spec.Object == "" {
-		spec.Object = "latest"
-	}
-
-	resolver := docker.NewResolver(docker.ResolverOptions{})
-
-	store := content.NewFile("")
-	defer store.Close()
-
-	// Get the filename from path and use it as the name in the file store
-	name := filepath.Base(path)
-
-	desc, err := store.Add(name, layerMediaType, path)
+	im, err := oras.NewImageFromSIF(path, types.MediaType(layerMediaType))
 	if err != nil {
-		return fmt.Errorf("unable to add SIF to store: %w", err)
+		return err
 	}
 
-	manifest, manifestDesc, config, configDesc, err := content.GenerateManifestAndConfig(nil, nil, desc)
-	if err != nil {
-		return fmt.Errorf("unable to generate manifest and config: %w", err)
-	}
-
-	if err := store.Load(configDesc, config); err != nil {
-		return fmt.Errorf("unable to load config: %w", err)
-	}
-
-	if err := store.StoreManifest(spec.String(), manifestDesc, manifest); err != nil {
-		return fmt.Errorf("unable to store manifest: %w", err)
-	}
-
-	if _, err := oras.Copy(context.Background(), store, spec.String(), resolver, ""); err != nil {
-		return fmt.Errorf("unable to push: %w", err)
-	}
-
-	return nil
+	return remote.Write(ir, im, remote.WithUserAgent("singularity e2e-test"))
 }
 
 func (c ctx) testPullDisableCacheCmd(t *testing.T) {
@@ -499,21 +693,48 @@ func (c ctx) testPullDisableCacheCmd(t *testing.T) {
 		name      string
 		imagePath string
 		imageSrc  string
+		platform  string
+		oci       bool
+		noHTTPS   bool
 	}{
 		{
-			name:      "library",
-			imagePath: filepath.Join(c.env.TestDir, "library.sif"),
+			name:      "library native",
+			imagePath: filepath.Join(c.env.TestDir, "nocache-library.sif"),
 			imageSrc:  "library://alpine:latest",
 		},
+		{
+			name:      "library oci-sif",
+			imagePath: filepath.Join(c.env.TestDir, "nocache-library.oci.sif"),
+			imageSrc:  "library://sylabs/tests/alpine-oci-sif:latest",
+			platform:  "linux/amd64",
+			oci:       true,
+		},
 		{
 			name:      "oras",
-			imagePath: filepath.Join(c.env.TestDir, "oras.sif"),
+			imagePath: filepath.Join(c.env.TestDir, "nocache-oras.sif"),
 			imageSrc:  "oras://" + c.env.TestRegistry + "/pull_test_sif:latest",
 		},
+		{
+			name:      "docker oci-sif",
+			imagePath: filepath.Join(c.env.TestDir, "nocache-docker.oci.sif"),
+			imageSrc:  c.env.TestRegistryImage,
+			oci:       true,
+			noHTTPS:   true,
+		},
 	}
 
 	for _, tt := range disableCacheTests {
-		cmdArgs := []string{"--disable-cache", tt.imagePath, tt.imageSrc}
+		cmdArgs := []string{"--disable-cache"}
+		if tt.oci {
+			cmdArgs = append(cmdArgs, "--oci")
+		}
+		if tt.noHTTPS {
+			cmdArgs = append(cmdArgs, "--no-https")
+		}
+		if tt.platform != "" {
+			cmdArgs = append(cmdArgs, "--platform", tt.platform)
+		}
+		cmdArgs = append(cmdArgs, tt.imagePath, tt.imageSrc)
 		c.env.RunSingularity(
 			t,
 			e2e.AsSubtest(tt.name),
