@@ -5,9 +5,9 @@
 
 package singularity
 
-import "github.com/sylabs/singularity/internal/pkg/plugin"
+import "github.com/sylabs/singularity/v4/internal/pkg/plugin"
 
 // DisablePlugin disables the named plugin.
-func DisablePlugin(name, libexecdir string) error {
+func DisablePlugin(name, _ string) error {
 	return plugin.Disable(name)
 }
