@@ -10,7 +10,7 @@ import (
 	"io"
 	"os"
 
-	"github.com/sylabs/singularity/internal/pkg/remote"
+	"github.com/sylabs/singularity/v4/internal/pkg/remote"
 )
 
 // RemoteRemove deletes a remote endpoint from the configuration
