@@ -1,4 +1,4 @@
-// Copyright (c) 2022-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) 2022-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -22,13 +22,12 @@ import (
 	"syscall"
 
 	"github.com/google/uuid"
-	lccgroups "github.com/opencontainers/runc/libcontainer/cgroups"
 	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/samber/lo"
 	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
 	"github.com/sylabs/singularity/v4/internal/pkg/cache"
 	"github.com/sylabs/singularity/v4/internal/pkg/cgroups"
-	ociclient "github.com/sylabs/singularity/v4/internal/pkg/client/oci"
+	"github.com/sylabs/singularity/v4/internal/pkg/ociimage"
 	"github.com/sylabs/singularity/v4/internal/pkg/runtime/launcher"
 	"github.com/sylabs/singularity/v4/internal/pkg/util/env"
 	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
@@ -192,10 +191,6 @@ func checkOpts(lo launcher.Options) error {
 		badOpt = append(badOpt, "ContainAll")
 	}
 
-	if lo.AppName != "" {
-		badOpt = append(badOpt, "AppName")
-	}
-
 	if lo.KeyInfo != nil {
 		badOpt = append(badOpt, "KeyInfo")
 	}
@@ -307,6 +302,14 @@ func (l *Launcher) finalizeSpec(ctx context.Context, b ocibundle.Bundle, spec *s
 		return fmt.Errorf("bundle has no image spec")
 	}
 
+	if len(imgSpec.Config.Volumes) != 0 {
+		sylog.Warningf("Container specifies volume mounts, which are not supported by singularity. It may not operate as expected.")
+	}
+	if len(imgSpec.Config.ExposedPorts) != 0 && !l.cfg.Namespaces.Net {
+		sylog.Infof("Container specifies exposed ports.")
+		sylog.Infof("Unless started with --net, singularity uses the host network namespace and all ports are exposed subject to host firewall rules.")
+	}
+
 	// In the absence of a USER in the OCI image config, we will run the
 	// container process as our own user / group, i.e. the uid / gid outside of
 	// any initial id-mapped user namespace.
@@ -325,7 +328,7 @@ func (l *Launcher) finalizeSpec(ctx context.Context, b ocibundle.Bundle, spec *s
 	containerUser := false
 
 	// If the OCI image config specifies a USER we will:
-	//  * When unprivileged - run as that user, via nested subuid/gid mappings (host user -> userns root -> OCI USER)
+	//  * When unprivileged - run as that user, via nested subuid/gid mappings.
 	//  * When privileged - directly run as that user, as a host uid/gid.
 	if imgSpec.Config.User != "" {
 		imgUser, err := tools.BundleUser(b.Path(), imgSpec.Config.User)
@@ -353,7 +356,7 @@ func (l *Launcher) finalizeSpec(ctx context.Context, b ocibundle.Bundle, spec *s
 	}
 
 	if targetUID != 0 && currentUID != 0 {
-		uidMap, gidMap, err := getReverseUserMaps(currentUID, targetUID, targetGID)
+		uidMap, gidMap, err := getReverseUserMaps(currentUID, uint32(rootlessUID), uint32(rootlessGID))
 		if err != nil {
 			return err
 		}
@@ -408,6 +411,10 @@ func (l *Launcher) finalizeSpec(ctx context.Context, b ocibundle.Bundle, spec *s
 
 	l.handleVarTmpToTmpSymlink(spec)
 
+	if err := addAnnotations(spec, imgSpec); err != nil {
+		return err
+	}
+
 	return b.Update(ctx, spec)
 }
 
@@ -667,8 +674,8 @@ func (l *Launcher) Exec(ctx context.Context, ep launcher.ExecParams) error {
 		return fmt.Errorf("%w: instanceName", ErrNotImplemented)
 	}
 
-	if l.cfg.SysContext == nil {
-		return fmt.Errorf("launcher SysContext must be set for OCI image handling")
+	if l.cfg.TransportOptions == nil {
+		return fmt.Errorf("launcher TransportOptions must be set for OCI image handling")
 	}
 
 	// Handle bare image paths and check image file format.
@@ -738,7 +745,7 @@ func (l *Launcher) Exec(ctx context.Context, ep launcher.ExecParams) error {
 		b, err = native.New(
 			native.OptBundlePath(bundleDir),
 			native.OptImageRef(image),
-			native.OptSysCtx(l.cfg.SysContext),
+			native.OptTransportOptions(l.cfg.TransportOptions),
 			native.OptImgCache(imgCache),
 		)
 	}
@@ -762,8 +769,10 @@ func (l *Launcher) Exec(ctx context.Context, ep launcher.ExecParams) error {
 	// Execution of runc/crun run, wrapped with overlay prep / cleanup.
 	err = l.RunWrapped(ctx, id.String(), b.Path(), "")
 
-	// Unmounts pristine rootfs from bundle, and removes the bundle.
-	if cleanupErr := b.Delete(ctx); cleanupErr != nil {
+	// Unmounts pristine rootfs from bundle, and removes the bundle. We want to
+	// make a best effort here even if the main context has been canceled, hence
+	// the use of context.Background().
+	if cleanupErr := b.Delete(context.Background()); cleanupErr != nil { //nolint:contextcheck
 		sylog.Errorf("Couldn't cleanup bundle: %v", err)
 	}
 
@@ -800,36 +809,34 @@ func (l *Launcher) RunWrapped(ctx context.Context, containerID, bundlePath, pidF
 			if err := os.MkdirAll(im.GetMountPoint(), 0o755); err != nil {
 				return err
 			}
-			if err := im.Mount(); err != nil {
+			if err := im.Mount(ctx); err != nil {
 				return err
 			}
 		}
 
-		// On cgroups v1 rootless, it's not possible to use systemd to manage cgroups.
-		// runc will fail if requested, so don't request it.
 		systemdCgroups := l.singularityConf.SystemdCgroups
-		uid, err := rootless.Getuid()
-		if err != nil {
-			return err
-		}
-		if uid != 0 && !lccgroups.IsCgroup2UnifiedMode() {
+		// If singularity.conf is set to use systemd for cgroup management, but
+		// we cannot due faulty configuration / environment (e.g. no Dbus),
+		// don't ask runc/crun to use systemd.
+		if systemdCgroups && !cgroups.CanUseCgroups(true, false) {
+			sylog.Infof("System configuration does not support cgroup management - starting container in current cgroup")
 			systemdCgroups = false
 		}
 
 		err = Run(ctx, containerID, absBundle, pidFile, systemdCgroups)
 
 		for _, im := range l.imageMountsByMountpoint {
-			im.Unmount()
+			im.Unmount(ctx)
 		}
 
 		return err
 	}
 
 	if len(l.cfg.OverlayPaths) > 0 {
-		return WrapWithOverlays(runFunc, absBundle, l.cfg.OverlayPaths, l.cfg.AllowSUID)
+		return WrapWithOverlays(ctx, runFunc, absBundle, l.cfg.OverlayPaths, l.cfg.AllowSUID)
 	}
 
-	return WrapWithWritableTmpFs(runFunc, absBundle, l.cfg.AllowSUID)
+	return WrapWithWritableTmpFs(ctx, runFunc, absBundle, l.cfg.AllowSUID)
 }
 
 // getCgroup will return a cgroup path and resources for the runtime to create.
@@ -837,6 +844,11 @@ func (l *Launcher) getCgroup() (path string, resources *specs.LinuxResources, er
 	if l.cfg.CGroupsJSON == "" {
 		return "", nil, nil
 	}
+
+	if !cgroups.CanUseCgroups(l.singularityConf.SystemdCgroups, true) {
+		return "", nil, fmt.Errorf("system configuration does not support cgroup management")
+	}
+
 	path = cgroups.DefaultPathForPid(l.singularityConf.SystemdCgroups, -1)
 	resources, err = cgroups.UnmarshalJSONResources(l.cfg.CGroupsJSON)
 	if err != nil {
@@ -874,7 +886,7 @@ func CrunNestCgroup() error {
 		return fmt.Errorf("singularity configuration is not initialized")
 	}
 
-	r, err := runtime()
+	r, err := Runtime()
 	if err != nil {
 		return err
 	}
@@ -889,9 +901,10 @@ func CrunNestCgroup() error {
 		return nil
 	}
 
-	// We can only create a new cgroup under cgroups v2 with systemd as manager.
-	// Generally we won't hit the issue that needs a workaround under cgroups v1, so no-op instead of a warning here.
-	if !(lccgroups.IsCgroup2UnifiedMode() && c.SystemdCgroups) {
+	// If rootless cgroup management is not possible for any reason, don't attempt to apply the workaround.
+	// It won't generally be needed in this case, as crun can't do any cgroups hanling either.
+	if !cgroups.CanUseCgroups(c.SystemdCgroups, false) {
+		sylog.Debugf("Skipping crun workaround - system configuration does not support cgroup management.")
 		return nil
 	}
 
@@ -916,7 +929,7 @@ func normalizeImageRef(imageRef string) (string, error) {
 	// We can't just look for a `<transport>:<path>` pair as bare filenames can contain colons.
 	// If we don't match a supported oci source transport, assume we have a bare filename.
 	parts := strings.SplitN(imageRef, ":", 2)
-	if len(parts) == 2 && ociclient.IsSupported(parts[0]) != "" {
+	if len(parts) == 2 && ociimage.SupportedTransport(parts[0]) != "" {
 		return imageRef, nil
 	}
 
