@@ -13,12 +13,12 @@ import (
 	"strings"
 	"syscall"
 
-	"github.com/sylabs/singularity/internal/pkg/util/bin"
-	"github.com/sylabs/singularity/internal/pkg/util/env"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/capabilities"
-	"github.com/sylabs/singularity/pkg/util/slice"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/env"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/capabilities"
+	"github.com/sylabs/singularity/v4/pkg/util/slice"
 )
 
 var (
@@ -195,7 +195,6 @@ func NVCLIEnvToFlags(nvidiaEnv []string) (flags []string, err error) {
 		if pair[0] == "NVIDIA_DISABLE_REQUIRE" {
 			disableRequire = true
 		}
-
 	}
 
 	if defaultDriverCaps {
