@@ -8,7 +8,7 @@ package sysctl
 import (
 	"testing"
 
-	"github.com/sylabs/singularity/internal/pkg/test"
+	"github.com/sylabs/singularity/v4/internal/pkg/test"
 )
 
 func TestGetSet(t *testing.T) {
